/* Copyright (c) 2009, Guillaume Delhumeau & Sophie Poullain
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Guillaume Delhumeau, Sophie Poullain nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package ecosysteme.model;

import java.awt.Graphics;

/**
 * Représente la carte sur laquelle se déroule la simulation.
 * @author Guillaume Delhumeau & Sophie Poullain
 */
public class Map
{

	private Case[][] map;
	public final static int NB_COLS = 100;
	public final static int NB_LINES = 100;
	public final static int CASE_WIDTH = 10;
	public final static int CASE_HEIGHT = 10;
	public final static int TAILLE_RESERVE = 18;
	public final static int NB_RESERVES = 3;
	private int nbFish;
	private int nbShark;
	private boolean isInit;

	public Map()
	{
		map = new Case[NB_COLS][NB_LINES];
		isInit = false;
	}

	void init()
	{

		for (int x = 0; x < NB_COLS; ++x)
		{
			for (int y = 0; y < NB_LINES; ++y)
			{
				map[x][y] = new Case();
			}
		}

		for (int i = 0; i < Parameters.SAFE_ZONES_NUMBER; ++i)
		{
			int midX = (int) (Math.random() * NB_COLS);
			int midY = (int) (Math.random() * NB_LINES);

			for (int x = midX - Parameters.SAFE_ZONES_SIZE; x < midX + Parameters.SAFE_ZONES_SIZE; ++x)
			{
				for (int y = midY - Parameters.SAFE_ZONES_SIZE; y < midY + Parameters.SAFE_ZONES_SIZE; ++y)
				{

					if (isCorrectCase(x, y))
					{
						if (Math.pow(x - midX, 2) + Math.pow(y - midY, 2) < Parameters.SAFE_ZONES_SIZE * Parameters.SAFE_ZONES_SIZE)
						{
							map[x][y].setType(Case.TYPE_PROTECTED);

						}
					}
				}
			}
		}
		createFishes();

		isInit = true;
	}

	public boolean isInit()
	{
		return isInit;
	}

	public void setIsInit(boolean isInit)
	{
		this.isInit = isInit;
	}

	public int getNbFish()
	{
		return nbFish;
	}

	public int getNbShark()
	{
		return nbShark;
	}

	public void createFishes()
	{
		for (int x = 0; x < NB_COLS; ++x)
		{
			for (int y = 0; y < NB_LINES; ++y)
			{
				if (Math.random() > 0.95)
				{
					Fish fish = new Fish(this);
					map[x][y].setFish(fish);
					fish.setX(x);
					fish.setY(y);
					fish.setAge((int) (Math.random() * Parameters.FISH_SHELF_LIFE));
				}
			}
		}
	}

	public void createShark()
	{
		//créer un nouveau requin a une place aléatoire sur la map de coordonnées (x,y)
		Shark shark = new Shark(this);
		int x2 = (int) (Math.random() * NB_COLS);
		int y2 = (int) (Math.random() * NB_LINES);
		while (map[x2][y2].getType() == Case.TYPE_PROTECTED)
		{
			x2 = (int) (Math.random() * NB_COLS);
			y2 = (int) (Math.random() * NB_LINES);
		}
		map[x2][y2].setShark(shark);
		shark.setX(x2);
		shark.setY(y2);
	}

	public void draw(Graphics g)
	{
		// We draw cases
		for (int c = 0; c < NB_COLS; ++c)
		{
			for (int l = 0; l < NB_LINES; ++l)
			{
				map[c][l].draw(g, c * CASE_WIDTH, l * CASE_HEIGHT);
			}
		}

		// we draw sharks
		for (int c = 0; c < NB_COLS; ++c)
		{
			for (int l = 0; l < NB_LINES; ++l)
			{
				Shark shark = map[c][l].getShark();
				if (shark != null)
				{
					shark.draw(g);
				}
			}
		}
	}

	public void update(int iter)
	{
		for (int x = 0; x <
						NB_COLS; ++x)
		{
			for (int y = 0; y <
							NB_LINES; ++y)
			{
				map[x][y].update(iter);
			}
		}

		// we count fishes & shark
		nbFish = 0;
		nbShark = 0;
		for (int x = 0; x < NB_COLS; ++x)
		{
			for (int y = 0; y < NB_LINES; ++y)
			{
				if (map[x][y].getFish() != null)
				{
					nbFish++;
				}
				if (map[x][y].getShark() != null)
				{
					nbShark++;
				}
			}
		}

	}

	public Case getCase(
					int c, int l)
	{
		return map[c][l];
	}

	public boolean isCorrectCase(int c, int l)
	{
		return c >= 0 && c < NB_COLS && l >= 0 && l < NB_LINES;
	}
}
