/* Copyright (c) 2009, Guillaume Delhumeau & Sophie Poullain
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Guillaume Delhumeau, Sophie Poullain nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package ecosysteme.model;

import ecosysteme.gui.Display;
import ecosysteme.*;
import ecosysteme.gui.Statistique;
import java.awt.Color;
import java.awt.Graphics;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * L'algorithme principal se situe dans cette classe. Elle est exécutable 
 * dans un Thread.
 * @author Guillaume Delhumeau & Sophie Poullain
 */
public class Simulation implements Runnable
{

	private Display display;
	private boolean running;
	private Map map;
	private int nbFrame;
	private long nextSecond;
	private int frameRate;
	private boolean pause;
	private Thread thread;
	private Statistique stats;

	public Simulation()
	{
		display = new Display();
		map = new Map();
		stats = new Statistique();
	}

	public Display getDisplay()
	{
		return display;
	}

	public void init(boolean newMap)
	{
		Shark.loadGfx();
		Fish.loadGfx();
		if (!map.isInit() || newMap)
		{
			map.init();
		}
		stats.init();
		draw();
	}

	public void start()
	{
		if (!running)
		{
			thread = new Thread(this);
			thread.start();
		}
	}

	public void stop()
	{
		running = false;
		pause = true;
		try
		{
			thread.join();
		} catch (InterruptedException ex)
		{
			Logger.getLogger(Simulation.class.getName()).log(Level.SEVERE, null, ex);
		}
	}

	public void createFish()
	{
		map.createFishes();
	}

	public void createShark()
	{
		map.createShark();
	}

	/**
	 * Méthode principale
	 */
	public void run()
	{
		int iter = 0;
		nextSecond = System.currentTimeMillis() + 1000;
		running = true;
		pause = false;
		while (running)
		{
			if (!pause)
			{
				nbFrame++;
				iter++;
				if (System.currentTimeMillis() > nextSecond)
				{
					frameRate = nbFrame;
					nbFrame = 0;
					nextSecond += 1000;
				}
				update(iter);
				draw();
				stats.update(iter, map.getNbFish(), map.getNbShark());
			}
			else
			{
				try
				{
					Thread.sleep(300);
				} catch (InterruptedException ex)
				{
					Logger.getLogger(Simulation.class.getName()).log(Level.SEVERE, null, ex);
				}
			}
		}

	}

	private void draw()
	{
		Graphics g = display.getMapGraphics();
		g.setColor(Color.blue);
		g.fillRect(0, 0, display.getWidth(), display.getHeight());
		map.draw(g);

		display.display();
	}

	private void update(int iter)
	{
		map.update(iter);
	}

	public boolean isPause()
	{
		return pause;
	}

	public void setPause(boolean pause)
	{
		this.pause = pause;
	}

	public boolean isRunning()
	{
		return running;
	}

	public Statistique getStats()
	{
		return stats;
	}

	public void setStats(Statistique stats)
	{
		this.stats = stats;
	}
}
